# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - PostMessage Action

    @copyright: 2012, 2013, 2014 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.Page import Page
from MoinMoin.PageEditor import PageEditor
from MoinSupport import getMetadata, writeHeaders
from MoinMessage import is_collection, to_replace, to_store, get_update_action, \
                        as_string
from MoinMessageSupport import MoinMessageAction

Dependencies = ['pages']

class PostMessage(MoinMessageAction):

    "A posted message handler."

    def handle_message_object(self, message):
        self.do_as_user(self.new_user, self._handle_message_object, [message])

    def _handle_message_object(self, message):

        "Handle the given 'message' object."

        request = self.request

        # Handle each update.

        all_successful = True
        any_successful = False

        for update in message.updates:

            # Handle a single part.

            if not is_collection(update):
                success = self.handle_message_parts(message, [update], update)

            # Or a collection of alternative representations for a single
            # update.

            else:
                success = self.handle_message_parts(message, update.get_payload(), update)

            all_successful = all_successful and success
            any_successful = any_successful or success

        # Default output.

        if any_successful:
            writeHeaders(request, "text/plain", getMetadata(self.page), "200 OK")
        else:
            writeHeaders(request, "text/plain", getMetadata(self.page), "403 Forbidden")

        if all_successful:
            request.write("All updates were successful.")
        elif any_successful:
            request.write("Some updates were unsuccessful.")
        else:
            request.write("No updates were successful.")

    def handle_message_parts(self, message, parts, update):

        """
        From the given 'message', handle the given 'parts', using the original
        'update' to determine whether the content is to replace or update page
        content, or whether it will be placed in a message store.
        """

        request = self.request

        # Test for privileges to change the page or message store.

        update_action = get_update_action(update)

        if not self.can_perform_action(update_action):
            return False

        # Handle the different update actions.
        # Update a message store for the page.

        if to_store(update):

            # Add any recipient from the original message.

            if self.message.has_key("To") and not update.has_key("To"):
                update["To"] = self.message["To"]

            # Add any authenticated user.
            # Note that where messages are signed by the real author, encrypted,
            # and then signed for sending, the authenticated user here is not
            # the real author.

            update["Moin-User"] = request.user and request.user.valid and request.user.name or None

            # Propagate time information using the unintuitive time functions.

            if message.date:
                update["Date"] = message.date.as_RFC2822_datetime_string()

            self.store.append(as_string(update))

        # Update the page.

        else:
            # NOTE: Should either choose preferred content types or somehow retain them
            # NOTE: all but present one at a time.

            body = []
            replace = to_replace(update)

            for part in parts:
                mimetype = part.get_content_type()
                encoding = part.get_content_charset()
                if mimetype == "text/moin":
                    payload = part.get_payload(decode=True)
                    body.append(encoding and unicode(payload, encoding) or payload)
                    if replace:
                        break

            if not replace:
                body.append(self.page.get_raw_body())

            page_editor = PageEditor(request, self.pagename)
            page_editor.saveText("\n\n".join(body), 0)

            # Refresh the page.

            self.page = Page(request, self.pagename)

        return True

# Action function.

def execute(pagename, request):
    PostMessage(pagename, request).do_action() # instead of render

# vim: tabstop=4 expandtab shiftwidth=4
